/**
 * Sample code for "Untestable Code" serie. Find more on
 * http://kaczanowscy.pl/tomek.
 * 
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including * without limitation the
 * rights to use, copy, modify, merge, publish, distribute, sublicense, and/or
 * sell copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 * 
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 * 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
package pl.kaczanowscy.uc.domain;

import static org.jeasytest.external.Util.arg;
import static org.jeasytest.external.Util.on;

import java.math.BigDecimal;
import java.util.Collections;

import junit.framework.TestCase;

import org.jeasytest.annotations.ClassUnderTest;
import org.jeasytest.annotations.JEasyTest;

import pl.kaczanowscy.uc.infrastructure.Database;
import static org.easymock.classextension.EasyMock.*;

/**
 * @author Tomek Kaczanowski
 * 
 */
@ClassUnderTest(ServiceA.class)
public class ServiceATest extends TestCase {

	private EntityX entity;

	private ServiceA serviceA;
	private ServiceB servB;
	private final BigDecimal total = new BigDecimal("125.40");

	@Override
	protected void setUp() throws Exception {
		entity = new EntityX(1, "entityName", "entityCode");
		serviceA = new ServiceA();
		servB = createMock(ServiceB.class);
	}

	@JEasyTest
	public void testBusinessOperation() throws InvalidItemStatus {
		on(Database.class).expectStaticNonVoidMethod("find").with(
				arg("select item from EntityY item where item.someProperty=?"),
				arg("abc")).andReturn(Collections.EMPTY_LIST);
		on(ServiceB.class).expectEmptyConstructor().andReturn(servB);
		on(Database.class).expectStaticVoidMethod("save").with(arg(entity));
		expect(servB.computeTotal(Collections.EMPTY_LIST)).andReturn(total);
		replay(servB);
		serviceA.doBusinessOperationXyz(entity);
		verify(servB);
		assertEquals(total, entity.getTotal());
	}
}
